/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package net.ajaest.jdk.core.winHandlers;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.ajaest.jdk.core.main.JDKGUIEngine;
import net.ajaest.jdk.core.main.JDKOptions;
import net.ajaest.jdk.core.main.Messages;
import net.ajaest.jdk.data.kanji.KanjiTag;
import net.ajaest.jdk.data.kanji.Radical;
import net.ajaest.jdk.gui.auxi.KanjiInfoWindow;
import net.ajaest.jdk.gui.windows.KanjiInfoExtWindow;
import net.ajaest.jdk.gui.windows.KanjiInfoSimpleWindow;
import net.ajaest.jdk.gui.windows.RadicalInfoWindow;

/**
 * 
 * @author ajaest10
 */
public class KanjiInfoWH implements WinHandler {

	// TODO: improve window positioning system
	private JDKGUIEngine jdkGui;

	private List<KanjiInfoWindow> kanjiDisplaying;
	private Map<KanjiTag, KanjiInfoWindow> kanjiWindows;

	private Map<Integer, RadicalInfoWindow> radicalWindows;

	private final Dimension screen;
	// Subscreen dimension(from upper left corner of the screen) that can be
	// used to put a windows
	private final Dimension positionableScreen;

	// Horizontal offset for new windows
	private static final int X_SHIFT = 140;
	// Vertical offset for new windows
	private static final int Y_SHIFT = 140;
	// Default window width used to calculate positionable screen size- Must be
	// multiple of X_SHIFT
	private static final int WINDOW_WIDTH = 584;
	// Default window height used to calculate positionable screen size- Must be
	// multibple of Y_SHFOT
	private static final int WINDOW_HEIGHT = 224;

	public KanjiInfoWH(JDKGUIEngine jdkgui) {

		this.jdkGui = jdkgui;

		kanjiDisplaying = new ArrayList<KanjiInfoWindow>();
		kanjiWindows = new HashMap<KanjiTag, KanjiInfoWindow>();

		radicalWindows = new HashMap<Integer, RadicalInfoWindow>();

		screen = Toolkit.getDefaultToolkit().getScreenSize();

		int positionableWidth = ((int) (screen.getWidth() - WINDOW_WIDTH) / X_SHIFT) * X_SHIFT;
		int positionableHeigth = ((int) (screen.getHeight() - WINDOW_HEIGHT) / Y_SHIFT) * X_SHIFT;
		positionableScreen = new Dimension(positionableWidth, positionableHeigth);
	}

	public void invokeWindow(KanjiTag kt) {

		KanjiInfoWindow kif = kanjiWindows.get(kt);
		
		if (kif==null) {

			if (jdkGui.getOptions().getExtendedInfo())
				kif = new KanjiInfoExtWindow(this, kt, jdkGui.getOptions().getRomanized());
			else
				kif = new KanjiInfoSimpleWindow(this, kt, jdkGui.getOptions().getRomanized());

			Integer locIndex = getFirstIndexOfAvaibleSpace();

			kif.setLocation(getScreenPositionOfIndex(locIndex));

			kif.setVisible(true);

			if (locIndex < kanjiDisplaying.size())
				kanjiDisplaying.set(locIndex, kif);
			else
				kanjiDisplaying.add(kif);

			kanjiWindows.put(kt, kif);
		} else {
			kif.toFront();
		}

		jdkGui.getMainWinHandler().getWindow().toFront();
	}

	public void radicalInfoButtonPressed(Integer classicRadical, KanjiInfoWindow kiw) {

		if (classicRadical != null) {
			RadicalInfoWindow riw = radicalWindows.get(classicRadical);

			if (riw == null) {

				Point pos = kiw.getLocation();
				pos.setLocation(pos.getX() + X_SHIFT, pos.getY());

				riw = new RadicalInfoWindow(this, jdkGui.getJavaDikt().getRadicalInfos(classicRadical));
				riw.setLocation(pos);
				riw.setVisible(true);

				radicalWindows.put(classicRadical, riw);
			} else {
				riw.toFront();
			}
		}
	}

	public String getRadicalString(Integer rad) {

		String radString;
		if (rad != null) {
			Radical retRad = null;

			for (Radical r : jdkGui.getJavaDikt().getRadicalInfos(rad)) {
				if (r.getUnicode() != null) {
					retRad = r;
					break;
				}
			}

			if (retRad != null)
				radString = new String(Character.toChars(retRad.getUnicode()));
			else
				radString = rad.toString();
		} else
			radString = "--";

		return radString;
	}

	public void windowClosed(KanjiInfoWindow kif) {

		int closedIndex = kanjiDisplaying.indexOf(kif);

		if ((closedIndex + 1) == kanjiDisplaying.size())
			kanjiDisplaying.remove(closedIndex);
		else
			kanjiDisplaying.set(closedIndex, null);

		kanjiWindows.remove(kif.getKanjiTag());
	}

	public void windowClosed(RadicalInfoWindow riw) {

		radicalWindows.remove(riw.getRadNumber());
	}

	public JDKOptions getOptions() {
		return jdkGui.getOptions();
	}

	public String getMessage(String message) {
		return Messages.get(message);
	}

	public List<Image> getIcons() {

		return jdkGui.getIcons();
	}

	public Font getUnicodeFont() {

		return jdkGui.getUnicodeFont();
	}

	public Point getScreenPositionOfIndex(Integer index) {

		int xpos = (X_SHIFT * index) % ((int) positionableScreen.getWidth());
		int ypos = (((X_SHIFT * index) / ((int) positionableScreen.getWidth())) * Y_SHIFT) % ((int) positionableScreen.getHeight());

		Point position = new Point(xpos, ypos);

		return position;
	}

	public Integer getFirstIndexOfAvaibleSpace() {

		Integer index = kanjiDisplaying.indexOf(null);

		if (index == -1)
			index = kanjiDisplaying.size();

		return index;
	}

	public void disposeAllWindows() {

		for (RadicalInfoWindow riw : radicalWindows.values())
			riw.dispose();

		for (KanjiInfoWindow kiw : kanjiWindows.values())
			kiw.dispose();

		// kanjiDisplaying = new ArrayList<KanjiInfoWindow>();
		// kanjiWindows = new HashMap<KanjiTag, KanjiInfoWindow>();
		//
		// radicalWindows = new HashMap<Integer, RadicalInfoWindow>();
	}

	@Override
	protected void finalize() {
		disposeAllWindows();
	}
}
